@@ -2,7 +2,7 @@
 /*
  *  Authors: Jeffrey Stedfast <fejj@ximian.com>
  *
- *  Copyright 2001 Ximian, Inc. (www.ximian.com)
+ *  Copyright 2001-2002 Ximian, Inc. (www.ximian.com)
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -31,18 +31,36 @@ extern "C" {
 
 #include "gmime-filter.h"
 
+#define GMIME_TYPE_FILTER_FROM            (g_mime_filter_from_get_type ())
+#define GMIME_FILTER_FROM(obj)            (GMIME_CHECK_CAST ((obj), GMIME_TYPE_FILTER_FROM, GMimeFilterFrom))
+#define GMIME_FILTER_FROM_CLASS(klass)    (GMIME_CHECK_CLASS_CAST ((klass), GMIME_TYPE_FILTER_FROM, GMimeFilterFromClass))
+#define GMIME_IS_FILTER_FROM(obj)         (GMIME_CHECK_TYPE ((obj), GMIME_TYPE_FILTER_FROM))
+#define GMIME_IS_FILTER_FROM_CLASS(klass) (GMIME_CHECK_CLASS_TYPE ((klass), GMIME_TYPE_FILTER_FROM))
+#define GMIME_FILTER_FROM_GET_CLASS(obj)  (GMIME_CHECK_GET_CLASS ((obj), GMIME_TYPE_FILTER_FROM, GMimeFilterFromClass))
+
+typedef struct _GMimeFilterFrom GMimeFilterFrom;
+typedef struct _GMimeFilterFromClass GMimeFilterFromClass;
+
 typedef enum {
 	GMIME_FILTER_FROM_MODE_DEFAULT  = 0,
 	GMIME_FILTER_FROM_MODE_ESCAPE   = 0,
 	GMIME_FILTER_FROM_MODE_ARMOR    = 1,
 } GMimeFilterFromMode;
 
-typedef struct _GMimeFilterFrom {
-	GMimeFilter parent;
+struct _GMimeFilterFrom {
+	GMimeFilter parent_object;
 	
 	GMimeFilterFromMode mode;
 	gboolean midline;
-} GMimeFilterFrom;
+};
+
+struct _GMimeFilterFromClass {
+	GMimeFilterClass parent_class;
+	
+};
+
+
+GType g_mime_filter_from_get_type (void);
 
 GMimeFilter *g_mime_filter_from_new (GMimeFilterFromMode mode);
 
