@@ -33,14 +33,13 @@ extern "C" {
 
 typedef struct _GMimeHeader GMimeHeader;
 
-typedef void (*GMimeHeaderFunc) (const char *name, const char *value, gpointer data);
+typedef void (*GMimeHeaderForeachFunc) (const char *name, const char *value, gpointer user_data);
+typedef ssize_t (*GMimeHeaderWriteFunc) (GMimeStream *stream, const char *name, const char *value);
 
 GMimeHeader *g_mime_header_new (void);
 
 void g_mime_header_destroy (GMimeHeader *header);
 
-void g_mime_header_foreach (const GMimeHeader *header, GMimeHeaderFunc func, gpointer data);
-
 void g_mime_header_set (GMimeHeader *header, const char *name, const char *value);
 
 void g_mime_header_add (GMimeHeader *header, const char *name, const char *value);
@@ -49,10 +48,14 @@ const char *g_mime_header_get (const GMimeHeader *header, const char *name);
 
 void g_mime_header_remove (GMimeHeader *header, const char *name);
 
-void g_mime_header_write_to_stream (const GMimeHeader *header, GMimeStream *stream);
+ssize_t g_mime_header_write_to_stream (const GMimeHeader *header, GMimeStream *stream);
 
 char *g_mime_header_to_string (const GMimeHeader *header);
 
+void g_mime_header_foreach (const GMimeHeader *header, GMimeHeaderForeachFunc func, gpointer user_data);
+
+void g_mime_header_set_write_func (GMimeHeader *header, const char *name, GMimeHeaderWriteFunc func);
+
 #ifdef __cplusplus
 }
 #endif /* __cplusplus */
