@@ -33,7 +33,6 @@
 
 #include "gmime-multipart.h"
 #include "gmime-utils.h"
-#include "strlib.h"
 
 
 #define d(x) x
@@ -167,7 +166,7 @@ multipart_add_header (GMimeObject *object, const char *header, const char *value
 	/* Make sure that the header is a Content-* header, else it
            doesn't belong on a multipart */
 	
-	if (!strncasecmp ("Content-", header, 8))
+	if (!g_strncasecmp ("Content-", header, 8))
 		GMIME_OBJECT_CLASS (parent_class)->add_header (object, header, value);
 }
 
@@ -175,13 +174,13 @@ static void
 multipart_set_header (GMimeObject *object, const char *header, const char *value)
 {
 	/* RFC 1864 states that you cannot set a Content-MD5 on a multipart */
-	if (!strcasecmp ("Content-MD5", header))
+	if (!g_strcasecmp ("Content-MD5", header))
 		return;
 	
 	/* Make sure that the header is a Content-* header, else it
            doesn't belong on a multipart */
 	
-	if (!strncasecmp ("Content-", header, 8))
+	if (!g_strncasecmp ("Content-", header, 8))
 		GMIME_OBJECT_CLASS (parent_class)->set_header (object, header, value);
 }
 
@@ -191,7 +190,7 @@ multipart_get_header (GMimeObject *object, const char *header)
 	/* Make sure that the header is a Content-* header, else it
            doesn't belong on a multipart */
 	
-	if (!strncasecmp ("Content-", header, 8))
+	if (!g_strncasecmp ("Content-", header, 8))
 		return GMIME_OBJECT_CLASS (parent_class)->get_header (object, header);
 	else
 		return NULL;
@@ -203,8 +202,8 @@ multipart_remove_header (GMimeObject *object, const char *header)
 	/* Make sure that the header is a Content-* header, else it
            doesn't belong on a multipart */
 	
-	if (!strncasecmp ("Content-", header, 8))
-		return GMIME_OBJECT_CLASS (parent_class)->remove_header (object, header);
+	if (!g_strncasecmp ("Content-", header, 8))
+		GMIME_OBJECT_CLASS (parent_class)->remove_header (object, header);
 }
 
 static void
