@@ -407,22 +407,31 @@ get_time (const unsigned char *in, unsigned int inlen, int *hour, int *min, int
 static int
 get_tzone (struct _date_token **token)
 {
-	int i;
+	const unsigned char *inptr, *inend;
+	unsigned int inlen;
+	int i, t;
 	
 	for (i = 0; *token && i < 2; *token = (*token)->next, i++) {
-		const unsigned char *inptr = (*token)->start;
-		unsigned int inlen = (*token)->len;
+		inptr = (*token)->start;
+		inlen = (*token)->len;
+		inend = inptr + inlen;
 		
 		if (*inptr == '+' || *inptr == '-') {
 			return decode_int (inptr, inlen);
 		} else {
-			int t;
-			
-			if (*inptr == '(')
+			if (*inptr == '(') {
 				inptr++;
+				if (*(inend - 1) == ')')
+					inlen -= 2;
+				else
+					inlen--;
+			}
 			
 			for (t = 0; t < 15; t++) {
-				unsigned int len = MIN (strlen (tz_offsets[t].name), inlen - 1);
+				unsigned int len = strlen (tz_offsets[t].name);
+				
+				if (len != inlen)
+					continue;
 				
 				if (!strncmp (inptr, tz_offsets[t].name, len))
 					return tz_offsets[t].offset;
@@ -538,7 +547,7 @@ parse_broken_date (struct _date_token *tokens, int *tzone)
 	while (token) {
 		if (is_weekday (token) && !got_wday) {
 			if ((n = get_wday (token->start, token->len)) != -1) {
-				printf ("weekday; ");
+				d(printf ("weekday; "));
 				got_wday = TRUE;
 				tm.tm_wday = n;
 				goto next_token;
@@ -547,7 +556,7 @@ parse_broken_date (struct _date_token *tokens, int *tzone)
 		
 		if (is_month (token) && !got_month) {
 			if ((n = get_month (token->start, token->len)) != -1) {
-				printf ("month; ");
+				d(printf ("month; "));
 				got_month = TRUE;
 				tm.tm_mon = n;
 				goto next_token;
@@ -556,7 +565,7 @@ parse_broken_date (struct _date_token *tokens, int *tzone)
 		
 		if (is_time (token) && !tm.tm_hour && !tm.tm_min && !tm.tm_sec) {
 			if (get_time (token->start, token->len, &hour, &min, &sec)) {
-				printf ("time; ");
+				d(printf ("time; "));
 				tm.tm_hour = hour;
 				tm.tm_min = min;
 				tm.tm_sec = sec;
@@ -568,7 +577,7 @@ parse_broken_date (struct _date_token *tokens, int *tzone)
 			struct _date_token *t = token;
 			
 			if ((n = get_tzone (&t)) != -1) {
-				printf ("tzone; ");
+				d(printf ("tzone; "));
 				got_tzone = TRUE;
 				offset = n;
 				goto next_token;
@@ -578,23 +587,23 @@ parse_broken_date (struct _date_token *tokens, int *tzone)
 		if (is_numeric (token)) {
 			if (token->len == 4 && !tm.tm_year) {
 				if ((n = get_year (token->start, token->len)) != -1) {
-					printf ("year; ");
+					d(printf ("year; "));
 					tm.tm_year = n - 1900;
 					goto next_token;
 				}
 			} else {
 				if (!got_month && !got_wday && token->next && is_numeric (token->next)) {
-					printf ("mon; ");
+					d(printf ("mon; "));
 					n = decode_int (token->start, token->len);
 					got_month = TRUE;
 					tm.tm_mon = n - 1;
 					goto next_token;
 				} else if (!tm.tm_mday && (n = get_mday (token->start, token->len)) != -1) {
-					printf ("mday; ");
+					d(printf ("mday; "));
 					tm.tm_mday = n;
 					goto next_token;
 				} else if (!tm.tm_year) {
-					printf ("2-digit year; ");
+					d(printf ("2-digit year; "));
 					n = get_year (token->start, token->len);
 					tm.tm_year = n - 1900;
 					goto next_token;
@@ -602,14 +611,14 @@ parse_broken_date (struct _date_token *tokens, int *tzone)
 			}
 		}
 		
-		printf ("???; ");
+		d(printf ("???; "));
 		
 	next_token:
 		
 		token = token->next;
 	}
 	
-	printf ("\n");
+	d(printf ("\n"));
 		
 	time = mktime (&tm);
 #if defined(HAVE_TIMEZONE)
@@ -711,6 +720,158 @@ g_mime_utils_header_decode_date (const char *in, int *saveoffset)
 }
 
 
+/* external symbols from internet-address.c */
+extern void decode_lwsp (const char **in);
+extern char *decode_word (const char **in);
+extern char *decode_addrspec (const char **in);
+
+
+static char *
+decode_msgid (const char **in)
+{
+	const char *inptr = *in;
+	char *msgid = NULL;
+	
+	decode_lwsp (&inptr);
+	if (*inptr == '<') {
+		decode_lwsp (&inptr);
+		if ((msgid = decode_addrspec (&inptr))) {
+			decode_lwsp (&inptr);
+			if (*inptr != '>') {
+				w(g_warning ("Invalid msg-id; missing '>': %s", *in));
+			} else {
+				inptr++;
+			}
+			
+			*in = inptr;
+		} else {
+			w(g_warning ("Invalid msg-id; missing addr-spec: %s", *in));
+		}
+	} else {
+		w(g_warning ("Invalid msg-id; missing '<': %s", *in));
+	}
+	
+	return msgid;
+}
+
+
+/**
+ * g_mime_utils_decode_message_id:
+ * @message_id: string containing a message-id
+ *
+ * Decodes a msg-id as defined by rfc822.
+ *
+ * Returns the addr-spec portion of the msg-id.
+ **/
+char *
+g_mime_utils_decode_message_id (const char *message_id)
+{
+	g_return_val_if_fail (message_id != NULL, NULL);
+	
+	return decode_msgid (&message_id);
+}
+
+
+/**
+ * g_mime_references_decode:
+ * @text: string containing a list of msg-ids
+ *
+ * Decodes a list of msg-ids as in the References and/or In-Reply-To
+ * headers defined in rfc822.
+ *
+ * Returns a list of referenced msg-ids.
+ **/
+GMimeReferences *
+g_mime_references_decode (const char *text)
+{
+	GMimeReferences *refs, *tail, *ref;
+	const char *inptr = text;
+	char *word, *msgid;
+	
+	g_return_val_if_fail (text != NULL, NULL);
+	
+	refs = NULL;
+	tail = (GMimeReferences *) &refs;
+	
+	while (*inptr) {
+		decode_lwsp (&inptr);
+		if (*inptr == '<') {
+			/* looks like a msg-id */
+			if ((msgid = decode_msgid (&inptr))) {
+				ref = g_new (GMimeReferences, 1);
+				ref->next = NULL;
+				ref->msgid = msgid;
+				tail->next = ref;
+				tail = ref;
+			} else {
+				w(g_warning ("Invalid References header: %s", inptr));
+				break;
+			}
+		} else if (*inptr) {
+			/* looks like part of a phrase */
+			if ((word = decode_word (&inptr))) {
+				g_free (word);
+			} else {
+				w(g_warning ("Invalid References header: %s", inptr));
+				break;
+			}
+		}
+	}
+	
+	return refs;
+}
+
+
+/**
+ * g_mime_references_append:
+ * @refs: the address of a GMimeReferences list
+ * @msgid: a message-id string
+ *
+ * Appends a reference to msgid to the list of references.
+ **/
+void
+g_mime_references_append (GMimeReferences **refs, const char *msgid)
+{
+	GMimeReferences *ref;
+	
+	g_return_if_fail (refs != NULL);
+	g_return_if_fail (msgid != NULL);
+	
+	ref = (GMimeReferences *) refs;
+	while (ref->next)
+		ref = ref->next;
+	
+	ref->next = g_new (GMimeReferences, 1);
+	ref->next->msgid = g_strdup (msgid);
+	ref->next->next = NULL;
+}
+
+
+/**
+ * g_mime_references_clear:
+ * @refs: address of a GMimeReferences list
+ *
+ * Clears the GMimeReferences list and resets it to %NULL.
+ **/
+void
+g_mime_references_clear (GMimeReferences **refs)
+{
+	GMimeReferences *ref, *next;
+	
+	g_return_if_fail (refs != NULL);
+	
+	ref = *refs;
+	while (ref) {
+		next = ref->next;
+		g_free (ref->msgid);
+		g_free (ref);
+		ref = next;
+	}
+	
+	*refs = NULL;
+}
+
+
 /**
  * g_mime_utils_header_fold:
  * @in: input header string
